@@ -18,7 +18,7 @@ from gevent._greenlet_primitives import get_reachable_greenlets
 from suanpan import asyncio, error, g, runtime, utils
 from suanpan.arguments import Bool, BoolOrInt, Float, Int, String
 from suanpan.components import Arguments
-from suanpan.interfaces import HasDevMode, HasExitHooks, HasInitHooks, HasLogger
+from suanpan.interfaces import HasDevMode, HasExitHooks, HasInitHooks, HasLogger, HasEventHooks
 from suanpan.interfaces.optional import HasBaseServices
 from suanpan.log import logger
 from suanpan.mq import mq
@@ -33,7 +33,7 @@ from suanpan.path import rootDir
 
 
 class Stream(
-    HasBaseServices, HasLogger, HasDevMode, HasInitHooks, HasExitHooks, HasHandlers
+    HasBaseServices, HasLogger, HasDevMode, HasInitHooks, HasExitHooks, HasHandlers, HasEventHooks
 ):
     STREAM_ARGUMENTS = [
         String("stream-recv-queue", default=f"mq-{g.nodeId}"),
@@ -50,6 +50,8 @@ class Stream(
         Bool("stream-send-queue-trim-immediately", default=False),
         String("stream-sio-static-folder", default="/web"),
         String("stream-sio-static-url-prefix", default="/"),
+        String("stream-app-event-hub", default=f"app-event-hub-{g.appId}"),
+        Int("stream-app-event-hub-max-length", default=1000),
     ]
     STREAM_PARAMS_FILE = "params.json"
 
@@ -65,10 +67,11 @@ class Stream(
         self.triggerLoop = StreamTriggerLoop(self)
         self.sioLoop = StreamSIOLoop(self)
         self.httpLoop = StreamHTTPLoop(self)
+        self.pubsubLoop = StreamPubsubLoop(self)
 
     @property
     def loops(self):
-        return [self.callLoop, self.triggerLoop, self.sioLoop, self.httpLoop]
+        return [self.callLoop, self.triggerLoop, self.sioLoop, self.httpLoop, self.pubsubLoop]
 
     def beforeInit(self):
         logger.logDebugInfo()
@@ -395,7 +398,8 @@ class StreamSIOLoop(StreamBaseLoop):
 
     def __init__(self, stream):
         super().__init__(stream)
-        self.sio = socketio.Server(async_mode="gevent", cors_allowed_origins="*", json=json, max_http_buffer_size=100000000)
+        self.sio = socketio.Server(async_mode="gevent", cors_allowed_origins="*", json=json,
+                                   max_http_buffer_size=100000000)
         self.options = self._getDefaultOptions()
         self.extraStaticFiles = {}
         self.staticFiles = {}
@@ -609,3 +613,60 @@ class StreamHTTPLoop(StreamBaseLoop):
         self.server = gevent.pywsgi.WSGIServer(("", g.termPort), self.application)
         self._start_server()
         self.server.serve_forever()
+
+
+class StreamPubsubLoop(StreamBaseLoop):
+    def __init__(self, stream):
+        super().__init__(stream)
+
+    @property
+    def ready(self):
+        return True
+
+    def subscribe(self, **kwargs):
+        return mq.subscribeEvent(
+            self.stream.options["stream-app-event-hub"],
+            block=self.stream.options["stream-recv-queue-block"],
+            delay=self.stream.options["stream-recv-queue-delay"],
+            **kwargs,
+        )
+
+    def close(self):
+        logger.debug("Stream Pubsub Loop Closed")
+
+    @runtime.globalrun
+    def start(self):
+        logger.debug("Stream Pubsub Loop Starting...")
+        for event in self.subscribe():
+            _event = event.get("data", {})
+            self.handle(_event)
+
+    def handle(self, event):
+        logger.debug(f"get event: {event}")
+        topic = event.get('event_name')
+        if not topic:
+            return
+
+        data = event.get('data')
+        if not data:
+            return
+
+        try:
+            handler = self.stream.eventHooks.get(topic)
+            d = json.loads(data)
+            handler(d)
+        except Exception:
+            logger.error(f"handle event {topic} error:", exc_info=True)
+
+    @runtime.retry(stop_max_attempt_number=3)
+    def send(self, topic: str, payload: dict):
+        data = {"app_id": g.appId, "node_id": g.nodeId, "event_name": topic, "data": payload}
+        logger.debug(
+            utils.shorten(f"Send event: {data}", self.DEFAULT_LOGGER_MAX_LENGTH)
+        )
+        return mq.sendEvent(
+            self.stream.options["stream-app-event-hub"],
+            data,
+            maxlen=self.stream.options["stream-app-event-hub-max-length"],
+            trimImmediately=self.stream.options["stream-send-queue-trim-immediately"],
+        )
